@@ -17,22 +17,22 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 /**
  * SECTION:element-fdsrc
  * @see_also: #GstFdSink
  *
  * Read data from a unix file descriptor.
- * 
- * To generate data, enter some data on the console folowed by enter.
+ *
+ * To generate data, enter some data on the console followed by enter.
  * The above mentioned pipeline should dump data packets to the console.
- * 
+ *
  * If the #GstFdSrc:timeout property is set to a value bigger than 0, fdsrc will
  * generate an element message named
  * <classname>&quot;GstFdSrcTimeout&quot;</classname>
- * if no data was recieved in the given timeout.
+ * if no data was received in the given timeout.
  * The message's structure contains one field:
  * <itemizedlist>
  * <listitem>
@@ -43,7 +43,7 @@
  *   </para>
  * </listitem>
  * </itemizedlist>
- * 
+ *
  * <refsect2>
  * <title>Example launch line</title>
  * |[
@@ -51,8 +51,6 @@
  * ]| A simple pipeline to read from the standard input and dump the data
  * with a fakesink as hex ascii block.
  * </refsect2>
- * 
- * Last reviewed on 2008-06-20 (0.10.21)
  */
 
 #ifdef HAVE_CONFIG_H
@@ -160,8 +158,6 @@ gst_fd_src_class_init (GstFdSrcClass * klass)
    * GstFdSrc:timeout
    *
    * Post a message after timeout microseconds
-   *
-   * Since: 0.10.21
    */
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_TIMEOUT,
       g_param_spec_uint64 ("timeout", "Timeout",
@@ -635,13 +631,18 @@ gst_fd_src_uri_set_uri (GstURIHandler * handler, const gchar * uri,
 
   protocol = gst_uri_get_protocol (uri);
   if (strcmp (protocol, "fd") != 0) {
+    g_set_error (err, GST_URI_ERROR, GST_URI_ERROR_BAD_URI,
+        "Wrong protocol for fdsrc in uri: '%s'", uri);
     g_free (protocol);
     return FALSE;
   }
   g_free (protocol);
 
-  if (sscanf (uri, "fd://%d", &fd) != 1 || fd < 0)
+  if (sscanf (uri, "fd://%d", &fd) != 1 || fd < 0) {
+    g_set_error (err, GST_URI_ERROR, GST_URI_ERROR_BAD_URI,
+        "Bad file descriptor number in uri: '%s'", uri);
     return FALSE;
+  }
 
   if ((q = g_strstr_len (uri, -1, "?"))) {
     gchar *sp;
